<?php

namespace common\logic\Distribution\Distribution\Export;

use common\logic\AbstractGenerator;
use common\logic\Distribution\Distribution\Model\Distribution;
use common\logic\Order\Order\Model\Order;
use common\logic\Order\Order\Repository\OrderRepository;
use common\logic\Producer\Producer\Service\ProducerSolver;
use kartik\mpdf\Pdf;

class DistributionShoppingCartLabelsPdfGenerator extends AbstractGenerator implements DistributionExportGeneratorInterface
{
    protected ProducerSolver $producerSolver;
    protected OrderRepository $orderRepository;

    public function loadDependencies(): void
    {
        $this->producerSolver = $this->loadService(ProducerSolver::class);
        $this->orderRepository = $this->loadService(OrderRepository::class);
    }

    public function generate(Distribution $distribution, bool $save = false)
    {
        $ordersArray = $this->orderRepository->findOrdersByDistribution($distribution);
        $ordersArray = $this->filterOrdersExcludedUsersAndPointSales($ordersArray);

        $content = \Yii::$app->getView()->render('@backend/views/distribution/shopping-cart-labels.php', [
            'distribution' => $distribution,
            'ordersArray' => $ordersArray,
            'shoppingCartLabelsPerColumn' => $this->producerSolver->getConfig('export_shopping_cart_labels_number_per_column') ?: 8
        ]);

        $pdf = new Pdf([
            'mode' => Pdf::MODE_UTF8,
            'format' => Pdf::FORMAT_A4,
            'orientation' => Pdf::ORIENT_PORTRAIT,
            'destination' => Pdf::DEST_BROWSER,
            'filename' => \Yii::getAlias(
                '@app/web/pdf/Etiquettes-' . $distribution->date . '-' . $this->getProducerContextId() . '.pdf'
            ),
            'content' => $content,
            'cssInline' => $this->getCss(),
            'methods' => [
                'SetHeader' => ['Étiquettes du ' . date('d/m/Y', strtotime($distribution->date))],
                'SetFooter' => ['{PAGENO}'],
            ],
        ]);

        $pdf->getApi()->SetColumns(4);
        $pdf->getApi()->keepColumns = true;

        return $pdf->render();
    }

    public function filterOrdersExcludedUsersAndPointSales(array $ordersArray)
    {
        $ordersArrayFilter = [];

        foreach($ordersArray as $order) {
            if(!$this->isOrderUserExcludeExport($order)
                && !$this->isOrderPointSaleExcludeExport($order)) {
                $ordersArrayFilter[] = $order;
            }
        }

        return $ordersArrayFilter;
    }

    public function isOrderUserExcludeExport(Order $order): bool
    {
        return $order->user && $order->user->exclude_export_shopping_cart_labels;
    }

    public function isOrderPointSaleExcludeExport(Order $order): bool
    {
        return $order->pointSale && $order->pointSale->exclude_export_shopping_cart_labels;
    }

    public function getCss(): string
    {
        return '
            @page {
              margin: 0px 0px 0px 0px !important;
              padding: 0px 0px 0px 0px !important;
            }
            
            body {
              font-family: Arial, Helvetica, sans-serif;
              font-size: 13px;
            }
            
            .clr {
              clear: both;
            }
            
            .shopping-cart-label {
                -webkit-column-break-inside:avoid;
                column-break-inside: avoid;
                -webkit-page-break-inside:avoid;
                page-break-inside: avoid;
            }
            
            .shopping-cart-label .inner {
                   padding: 8px;
            }
            
            .shopping-cart-label .username {
                font-weight: bold;
                font-size: 13px;
            }
            
            .shopping-cart-label .point-sale {
                font-size: 10px;
                line-height: 12px;
                text-transform: uppercase;
                margin-bottom: 3px;
            }
            
            .shopping-cart-label .products {
                 font-size: 10px;
            }
                    ';
    }
}